@@ -8,11 +8,17 @@ fairness.profile.plot = function(response, predictors, sensitive, unfairness,
 
   # check the model to be fitted.
   check.label(model, fair.models, "model")
+  # remove optional arguments that do not belong after warning.
+  check.unused.args(model.args, fair.models.extra.args[[model]])
+  model.args =
+    model.args[names(model.args) %in% fair.models.extra.args[[model]]]
 
   # check response, predictors and sensitive attributes.
-  response = check.response(response, model = model)
-  predictors = check.data(predictors, nobs = length(response), varletter = "X")
-  sensitive = check.data(sensitive, nobs = length(response), varletter = "S")
+  response = check.response(response, model = model, family = model.args$family)
+  predictors =
+    check.data(predictors, nobs = sample.size(response), varletter = "X")
+  sensitive =
+    check.data(sensitive, nobs = sample.size(response), varletter = "S")
   unfairness = check.fairness.level(unfairness, scalar = FALSE)
 
   # check the type and legend of the plot.
@@ -22,11 +28,6 @@ fairness.profile.plot = function(response, predictors, sensitive, unfairness,
   if (!(model %in% models.for.plot[[type]]))
     stop("model ", q(model), " is not supported by profile plot ", q(type), ".")
 
-  # remove optional arguments that do not belong after warning.
-  check.unused.args(model.args, fair.models.extra.args[[model]])
-  model.args =
-    model.args[names(model.args) %in% fair.models.extra.args[[model]]]
-
   # check the cluster.
   cluster = check.cluster(cluster)
 
@@ -37,12 +38,17 @@ fairness.profile.plot = function(response, predictors, sensitive, unfairness,
                           sensitive = sensitive,
                           unfairness = level),
                      model.args))
-  }
+  }#MODEL.FIT
 
   fitted = smartSapply(cluster, unfairness, model.fit)
+  family = fitted[[1]]$main$family
 
   if (type == "coefficients") {
 
+    # these diagnostic plots require gridExtra as well to arrange panels.
+    if (family == "multinomial")
+      check.and.load.package("gridExtra")
+
     coefficients.plot(models = fitted, unfairness = unfairness,
       response = response, legend = legend)
 
@@ -57,9 +63,7 @@ fairness.profile.plot = function(response, predictors, sensitive, unfairness,
 
     if (model %in% fair.family) {
 
-      family = fitted[[1]]$main$family
-
-      if (family != "binomial")
+      if (!(family %in% c("binomial", "multinomial")))
         stop("model ", q(model), " with family ", q(family),
              " is not supported by profile plot ", q(type), ".")
 
@@ -93,55 +97,96 @@ coefficients.plot = function(models, unfairness, response, legend) {
   # extract the coefficients from the models.
   coefs = lapply(models, coef)
 
-  # arrange the coefficients (except the intercept, which is not meaningful
-  # in this context) in a data frame and a formula.
-  coefs = as.data.frame(do.call(rbind, coefs))
-  coefs = coefs[, names(coefs) != "(Intercept)", drop = FALSE]
-  lhs = paste0("`", names(coefs), "`", collapse = " + ")
-  coefs[, "unfairness"] = unfairness
-  formula = formula(paste(lhs, "~ unfairness"))
+  if (is.matrix(coefs[[1]])) {
 
-  if (legend) {
+    coef.names = setdiff(rownames(coefs[[1]]), "(Intercept)")
+    grouped.coefs = vector(ncol(coefs[[1]]), mode = "list")
+    for (i in seq_along(grouped.coefs)) {
 
-    # make sure the graphic device is initialized, otherwise it is impossible
-    # to compute the grphical width of the labels.
-    if (inherits(try(strwidth(""), silent = TRUE), "try-error"))
-      plot.new()
+      grouped.coefs[[i]] = do.call(rbind, lapply(coefs,
+                             function(cc) cc[rownames(cc) != "(Intercept)", i]))
+      grouped.coefs[[i]] = data.frame(grouped.coefs[[i]], unfairness)
 
-    label.width = strwidth(setdiff(names(coefs), "unfairness"))
-    x.axis.range = extended.range(coefs$unfairness)
-    xlim = c(x.axis.range[1], x.axis.range[2] + label.width)
-    x.at = pretty(x.axis.range)
-    x.at = x.at[(x.at >= x.axis.range[1]) & (x.at <= x.axis.range[2])]
+    }#FOR
+    names(grouped.coefs) = colnames(coefs[[1]])
 
   }#THEN
   else {
 
-    label.width = 0
-    xlim = extended.range(coefs$unfairness)
-    x.at = pretty(xlim)
-    x.at = x.at[(x.at >= xlim[1]) & (x.at <= xlim[2])]
+    coefs = as.data.frame(do.call(rbind, coefs))
+    coefs = coefs[, names(coefs) != "(Intercept)", drop = FALSE]
+    coef.names = names(coefs)
+    coefs[, "unfairness"] = unfairness
+    grouped.coefs = list(coefs)
 
   }#ELSE
 
-  # disable clipping to be able to draw the axis.
-  suppressWarnings(lattice::trellis.par.set("clip", list(panel = "off")))
+  panels = vector(length(grouped.coefs), mode = "list")
 
-  # plot and return (relying on autoprint to display the plot).
-  lattice::xyplot(formula, data = coefs,
-    xlab = "unfairness", ylab = "coefficients", type = "l",
-    scales = list(tck = c(1, 0), x = list(at = NULL)), xlim = xlim,
-    panel = function(y, x, ...) {
+  for (i in seq_along(grouped.coefs)) {
 
-      lattice::panel.xyplot(y = y, x = x, ...)
-      lattice::panel.abline(h = 0, col = "lightgray", lty = "dashed")
-      if (legend)
-        lattice::panel.text(x = max(x) + 0.01, y = y[x == max(x)], pos = 4,
-                   labels = setdiff(names(coefs), "unfairness"))
+    coefs = grouped.coefs[[i]]
 
-      lattice::panel.axis(side = "bottom", at = x.at, outside = TRUE)
+    # arrange the coefficients (except the intercept, which is not meaningful
+    # in this context) in a data frame and a formula.
+    lhs = paste0("`", coef.names, "`", collapse = " + ")
+    formula = formula(paste(lhs, "~ unfairness"))
 
-    })
+    if (legend) {
+
+      # make sure the graphic device is initialized, otherwise it is impossible
+      # to compute the grphical width of the labels.
+      if (inherits(try(strwidth(""), silent = TRUE), "try-error"))
+        plot.new()
+
+      label.width = strwidth(setdiff(coef.names, "unfairness"))
+      x.axis.range = extended.range(coefs$unfairness)
+      xlim = c(x.axis.range[1], x.axis.range[2] + label.width)
+      x.at = pretty(x.axis.range)
+      x.at = x.at[(x.at >= x.axis.range[1]) & (x.at <= x.axis.range[2])]
+
+    }#THEN
+    else {
+
+      label.width = 0
+      xlim = extended.range(coefs$unfairness)
+      x.at = pretty(xlim)
+      x.at = x.at[(x.at >= xlim[1]) & (x.at <= xlim[2])]
+
+    }#ELSE
+
+    # disable clipping to be able to draw the axis.
+    suppressWarnings(lattice::trellis.par.set("clip", list(panel = "off")))
+
+    # if there are multiple sets of coefficients, identify which one will be
+    # plotted in the y-axis label.
+    if (!is.null(names(grouped.coefs)))
+      ylab = paste("coefficients for", q(names(grouped.coefs)[i]))
+    else
+      ylab = "coefficients"
+
+    # plot and return (relying on autoprint to display the plot).
+    panels[[i]] = lattice::xyplot(formula, data = coefs,
+      xlab = "unfairness", ylab = ylab, type = "l",
+      scales = list(tck = c(1, 0), x = list(at = NULL)), xlim = xlim,
+      panel = function(y, x, ...) {
+
+        lattice::panel.xyplot(y = y, x = x, ...)
+        lattice::panel.abline(h = 0, col = "lightgray", lty = "dashed")
+        if (legend)
+          lattice::panel.text(x = max(x) + 0.01, y = y[x == max(x)], pos = 4,
+                     labels = setdiff(coef.names, "unfairness"))
+
+        lattice::panel.axis(side = "bottom", at = x.at, outside = TRUE)
+
+      })
+
+  }#FOR
+
+  if (length(panels) == 1)
+    panels[[1]]
+  else
+    do.call(gridExtra::grid.arrange, panels)
 
 }#COEFFICIENTS.PLOT
 
@@ -322,7 +367,10 @@ precision.recall.plot = function(models, unfairness, response, legend) {
   # extract the fitted values from the models.
   models = lapply(models, `[[`, "main")
   fitted = lapply(models, `[[`, "fitted.values")
-  fitted = lapply(fitted, prob2class, labels = levels(response))
+  if (nlevels(response) == 2)
+    fitted = lapply(fitted, linpred2class, labels = levels(response))
+  else
+    fitted = lapply(fitted, linpred2mclass, labels = levels(response))
 
   # compute precision and recall.
   pr = data.frame(unfairness = unfairness, precision = 0, recall = 0)
